<?php

/**
 * @file
 * Administration forms for managing form/field settings.
 */

/**
 * Initial admin form callback.
 */
function ac_attr_admin_settings() {
  $form = [];

  $about = [];
  $about[] = t('The forms listed here are added as people browse the site. If you want to set the <code>purpose</code> of a specific <code>field</code> on a specific <code>form</code> you can enter it here.');
  $about[] = t('If you want a <code>purpose</code> to be set for any <code>fieldname</code>, regardless of the <code>form</code>, set it on the Defaults page.');
  $form['about'] = [
    '#markup' => '<p>' . implode("</p>\n<p>", $about) . '<p>',
    '#prefix' => '<div class="messages info">',
    '#suffix' => '</div>',
  ];

  $form_list = [];
  // Fetch known forms.
  $forms = db_select('ac_attr_forms', 'f')
    ->fields('f')
    ->orderBy('form_id')
    ->execute();
  while ($row = $forms->fetchAssoc()) {
    // Prep the form values.
    $form_fields = ac_attr_get_fields_with_ac_value($row['aaid']);

    // Add the fieldset for this form.
    $form_list[] = 'form_list_' . $row['aaid'];
    $form['form_list_' . $row['aaid']] = [
      '#type' => 'fieldset',
      '#collapsible' => TRUE,
      '#description' => t('These are the available fields we can set autocomplete values for.'),
    ];
    // For easy access in the submit handler.
    $form['form_list_' . $row['aaid']]['aaid'] = [
      '#type' => 'hidden',
      '#value' => $row['aaid'],
    ];
    $form['form_list_' . $row['aaid']]['ignore_form_id'] = [
      '#type' => 'checkbox',
      '#title' => t('Ignore this form?'),
      '#default_value' => $row['ignore_form_id'],
      '#description' => t('Should we skip processing this form for the autocomplete attribute?'),
    ];
    $form['form_list_' . $row['aaid']]['fetch_fields_again'] = [
      '#type' => 'checkbox',
      '#title' => t('Fetch fields for this form again?'),
      '#default_value' => $row['fetch_fields_again'],
      '#description' => t('If this is checked the next time this form is displayed the form fields will be updated.  Any new fields will be added.'),
    ];
    $form['form_list_' . $row['aaid']]['fields'] = [
      '#type' => 'fieldset',
      '#title' => t('Fields'),
      '#collapsible' => TRUE,
    ];
    $collapse_fieldset = TRUE;
    $field_count = 0;
    $value_count = 0;
    foreach ($form_fields as $field_name => $value) {
      $field_count++;
      $form['form_list_' . $row['aaid']]['fields'][$field_name] = [
        '#type' => 'textfield',
        '#title' => t('Fieldname: @field_name', ['@field_name' => $field_name]),
        '#default_value' => $value,
      ];
      if (empty($value)) {
        $collapse_fieldset = FALSE;
      }
      else {
        $value_count++;
      }
    }
    // Collapse the Fields fieldset if all fields have values.
    $form['form_list_' . $row['aaid']]['fields']['#collapsed'] = $collapse_fieldset;
    // Update the Form fieldset with counts and collapsed state.
    $form['form_list_' . $row['aaid']]['#title'] = t('Form: @form (@value_count of @field_count)', [
      '@form' => $row['form_id'],
      '@field_count' => $field_count,
      '@value_count' => $value_count,
    ]);
    $form['form_list_' . $row['aaid']]['#collapsed'] = ($field_count == $value_count);
  }
  // For easy processing in the submit handler.
  $form['form_list'] = [
    '#type' => 'hidden',
    '#value' => $form_list,
  ];

  // Make our nested values work for the submit handler.
  $form['#tree'] = TRUE;

  // Add our submit buttons.
  $form['actions'] = [
    '#type' => 'actions',
    'submit' => [
      '#type' => 'submit',
      '#value' => t('Submit'),
    ],
  ];

  return $form;
}

/**
 * Submit handler for Admin Settings form.
 */
function ac_attr_admin_settings_submit($form, &$form_state) {
  $values = $form_state['values'];
  foreach ($values['form_list'] as $form_key) {
    $data = $values[$form_key];
    // Update form row.
    db_update('ac_attr_forms')
      ->fields([
        'ignore_form_id' => (int) $data['ignore_form_id'],
        'fetch_fields_again' => (int) $data['fetch_fields_again'],
      ])
      ->condition('aaid', (int) $data['aaid'])
      ->execute();
    // Update the fields.
    $fields = $data['fields'];
    foreach ($fields as $field_name => $value) {
      db_update('ac_attr_form_fields')
        ->fields([
          'value' => $value,
        ])
        ->condition('aaid', $data['aaid'])
        ->condition('field_name', $field_name)
        ->execute();
    }
  }
}

/**
 * Module configuration.
 */
function ac_attr_admin_config() {
  $form = [];

  $about = [];
  $about[] = t('This allows for the setting of default fields that <code>purposes</code> are allocated to.');
  $about[] = t('The <code>purpose</code> is the value that would be entered into the <code>autocomplete</code> attribute.');
  $about[] = t('If a field is not specifically set on the Form Settings page and then the values here will be used.');
  $form['about'] = [
    '#markup' => '<p>' . implode("</p>\n<p>", $about) . '<p>',
    '#prefix' => '<div class="messages info">',
    '#suffix' => '</div>',
  ];

  // Default purposes.
  $purposes = ac_attr_purposes();

  $config = [];
  $config_vals = variable_get('ac_attr_defaults', []);
  for ($i = 0; $i < count($config_vals); $i++) {
    $config[$config_vals[$i]['purpose']] = $config_vals[$i]['fields'];
  }

  for ($i = 0; $i < count($purposes); $i++) {
    $purpose = $purposes[$i]['purpose'];
    $desc = $purposes[$i]['description'];

    $fields = implode("\n", $purposes[$i]['fields']);
    if (!empty($config[$purpose])) {
      $fields = $config[$purpose];
    }

    $form['ac_attr_defaults'][$i] = [
      '#type' => 'fieldset',
      '#title' => $purpose,
      '#description' => $desc,
      '#collapsible' => TRUE,
      '#collapsed' => empty(trim($fields)),
    ];
    $form['ac_attr_defaults'][$i]['purpose'] = [
      '#type' => 'textfield',
      '#title' => t('Purpose keyword'),
      '#description' => t('The value entered/added to the autocomplete attribute'),
      '#default_value' => $purpose,
      '#required' => TRUE,
      '#disabled' => TRUE,
    ];
    $form['ac_attr_defaults'][$i]['fields'] = [
      '#type' => 'textarea',
      '#title' => t('Default fields'),
      '#default_value' => trim($fields),
    ];
  }

  // Make our nested values work for the submit handler.
  $form['#tree'] = TRUE;

  // Add our submit buttons.
  $form['actions'] = [
    '#type' => 'actions',
    'submit' => [
      '#type' => 'submit',
      '#value' => t('Submit'),
    ],
  ];
  $form['#submit'][] = 'ac_attr_admin_config_submit';

  return system_settings_form($form);
}

/**
 * We want to clear the cache for this data when this form is saved.
 */
function ac_attr_admin_config_submit($form, &$form_state) {
  // Clear the cache that this would live in.
  cache_clear_all('ac_attr_default_fields_ac:', 'cache', TRUE);
}
